 module.exports = (grunt) ->
  defaultTasks = [
    'clean'
    'coffeelint:mainCoffee'
    'handlebars:main'
    'coffee:main'
    'coffee:test'
    'coffeeify:test'
    'mocha'
  ]
  testTasks = [
    'clean'
    'coffeelint:mainCoffee'
    'handlebars:main'
    'coffee:main'
    'coffee:test'
    'coffeeify:test'
    'mocha'
  ]
  grunt.initConfig
    pkg: grunt.file.readJSON 'package.json'
    meta:
      banner: '\n/************************************************************************\n'+
        ' * <%= pkg.title || pkg.name %> - v<%= pkg.version %> - <%= grunt.template.today("yyyy-mm-dd") %> \n <%= pkg.homepage ? "* "+ pkg.homepage + "\n" : "" %>\n' +
        ' * Copyright (c) <%= grunt.template.today("yyyy") %> Banno LLC. All rights reserved. \n' +
        ' ***********************************************************************/\n'
    clean: ['lib/**', 'dist/**', 'test/*.js']
    coffee:
      options:
        map: true
        bare: false
      main:
        files: [
          {
            src: ['src/main/coffeescript/UploadItem.coffee']
            dest: 'dist/UploadItem.js'
          }
          {
            src: ['src/main/coffeescript/Uploader.coffee']
            dest: 'dist/Uploader.js'
          }
          {
            src: ['src/main/coffeescript/FileUploads.coffee']
            dest: 'dist/FileUploads.js'
          }
          {
            src: ['src/main/coffeescript/FileUpload.coffee']
            dest: 'dist/FileUpload.js'
          }
          {
            src: [
              'src/main/coffeescript/asset/model/AssetCollection.coffee'
            ]
            dest: 'dist/asset/model/AssetCollection.js'
          }
          {
            src: [
                'src/main/coffeescript/asset/model/AssetModel.coffee'
            ]
            dest: 'dist/asset/model/AssetModel.js'
          }
          {
            src: [
              'src/main/coffeescript/AssetCollectionAssetGrid.coffee'
            ]
            dest: 'dist/AssetCollectionAssetGrid.js'
          }
          {
            src: [
              'src/main/coffeescript/AssetGrid.coffee'
            ]
            dest: 'dist/AssetGrid.js'
          }
          {
            src: [
              'src/main/coffeescript/CommonHandlebarsHelpers.coffee'
            ]
            dest: 'dist/CommonHandlebarsHelpers.js'
          }
          {
            src: [
              'src/main/coffeescript/ContentTypeHelpers.coffee'
            ]
            dest: 'dist/ContentTypeHelpers.js'
          }
          {
            src: [
              'src/main/coffeescript/LocationDialogAssetGrid.coffee'
            ]
            dest: 'dist/LocationDialogAssetGrid.js'
          }
          {
            src: [
              'src/main/coffeescript/LocationsFileDialogView.coffee'
            ]
            dest: 'dist/LocationsFileDialogView.js'
          }
          {
            src: [
              'src/main/coffeescript/notify.coffee'
            ]
            dest: 'dist/notify.js'
          }
        ]
      test:
        files: [
          src:
            [
              'src/test/coffeescript/*.coffee'
              'src/test/coffeescript/**/*.coffee'
              'src/test/handlebars/**/*.coffee'
            ]
          dest:
            'lib/test.js'
        ]
    coffeelint:
      mainCoffee:
        src: [
          'Gruntfile.coffee'
          'src/main/coffeescript/*.coffee'
          'src/main/coffeescript/**/*.coffee'
        ]
        options:
          max_line_length:
            value: 80
            level: 'ignore'
          indentation:
            value: 2
            level: 'warn'
    coffeeify:
      test:
        files: [
          src:
            [
              'src/test/coffeescript/**/*.coffee'
              'src/test/coffeescript/**/*.hh.coffee'
              'src/test/handlebars/**/*.coffee'
              'src/test/handlebars/*.coffee'
            ]
          dest:
            'test/client-lib-specs.js'
        ]
    mocha:
      options:
        ignoreLeaks: true
        globals: [
          'window'
          'document'
        ]
        ui: 'bdd'
        run: true
        colors: true
        bail: false
        reporter: 'Min'
      all: ['test/**/*.html']
    handlebars:
      main:
        options:
          wrapped: true
          namespace: 'com.banno.clientlib.templates'
          node: true
          processName: (filename) ->
            filenameRegex = ///(?:.*/)*(.*)(?:\.handlebars$)///
            filenameRegex.exec(filename)[1]
        files: [
          src:
            [
              'src/main/handlebars/*.handlebars'
              'src/main/handlebars/**/*.handlebars'
              'src/main/handlebars/**/**/*.handlebars'
              'src/main/handlebars/**/**/**/*.handlebars'
            ]
          dest:
            'dist/templates.js'
        ]
    watch:
      options:
        debounceDelay: 250
      files: [
        'Gruntfile.coffee'
        'package.json'
        'src/**/*'
        'test/**/*.html'
      ]
      tasks: 'test'

  
  grunt.loadNpmTasks('grunt-contrib')
  grunt.loadNpmTasks('grunt-mocha')
  grunt.loadNpmTasks('grunt-coffeelint')
  grunt.loadNpmTasks('grunt-coffeeify')
  grunt.loadNpmTasks('grunt-bump')
  grunt.registerTask('interactive', ['watch'])
  grunt.registerTask('test', testTasks)
  grunt.registerTask('default', defaultTasks)