(function() {
  var ContentTypeHelpers, Handlebars, dateFormat;

  Handlebars = require('handlebars');

  dateFormat = this.dateFormat;

  ContentTypeHelpers = require('./ContentTypeHelpers.js').ContentTypeHelpers;

  Handlebars.registerHelper("thumbnail", function(id, contentType, w, h) {
    if (w == null) {
      w = 100;
    }
    if (h == null) {
      h = 100;
    }
    contentType = contentType.toLowerCase();
    if (contentType === "image/jpeg" || contentType === "image/png" || contentType === "image/gif" || contentType === "image/bmp") {
      w = isFinite(w) ? w : 100;
      h = isFinite(h) ? h : 100;
      return "/assets/files/" + id + "/thumbnail/" + w + "/" + h;
    } else {
      return "/assets/editor/img/file-types/" + ContentTypeHelpers.contentTypeToIcon(contentType);
    }
  });

  Handlebars.registerHelper("url", function(id) {
    return "/assets/files/" + id;
  });

  Handlebars.registerHelper("dateformat", function(date) {
    return new Handlebars.SafeString(dateFormat(new Date(date), "mm/dd/yyyy"));
  });

  Handlebars.registerHelper("datetimeformat", function(date) {
    return new Handlebars.SafeString(dateFormat(new Date(date), "ddd mmm dd yyyy h:MM tt"));
  });

  Handlebars.registerHelper("filesizeformat", function(filesize) {
    if (Math.pow(10, 6) > filesize) {
      return (Math.round(filesize / 10) / 100) + " KB";
    } else {
      return (Math.round(filesize / 10000) / 100) + " MB";
    }
  });

  Handlebars.registerHelper("filetype", function(contentTypeOrFilename) {
    var icon;

    if (contentTypeOrFilename.indexOf("/") < 0) {
      contentTypeOrFilename = ContentTypeHelpers.extensionToContentType(contentTypeOrFilename);
    }
    icon = ContentTypeHelpers.contentTypeToIcon(contentTypeOrFilename);
    icon = icon.slice(0, icon.indexOf('.')).toUpperCase();
    if (icon === "DEFAULT") {
      return "File";
    } else {
      return icon;
    }
  });

  Handlebars.registerHelper("getIcon", function(network) {
    if (network != null) {
      return network.replace(" ", "-").replace("!", "").toLowerCase();
    } else {
      return "branch";
    }
  });

  Handlebars.registerHelper("replaceSpace", function(text) {
    return text = text.replace(/\s/g, "-").replace("!", "").toLowerCase();
  });

  Handlebars.registerHelper("getLocationType", function(network) {
    if (network != null) {
      return "atm";
    } else {
      return "branch";
    }
  });

  Handlebars.registerHelper("roundNumber", function(num) {
    return num.toFixed(2);
  });

  Handlebars.registerHelper("isUnread", function(read) {
    if (read) {
      return "";
    } else {
      return "unread";
    }
  });

  Handlebars.registerHelper("userAvatar", function(userId) {
    return "/_/api/user/" + userId + "/image";
  });

}).call(this);
