(function() {
  var $, Backbone, _;

  $ = global.$;

  $ || ($ = require('jquery-browserify'));

  _ = require('underscore');

  Backbone = require('backbone');

  Backbone.$ = $;

  exports.FileUpload = Backbone.Model.extend({
    defaults: {
      name: "",
      type: "",
      size: 0,
      maxFileSize: 50000000,
      pending: false,
      progress: 0.00,
      complete: false,
      file: ""
    },
    initialize: function() {
      return this.listenTo(this, 'change:progress', this.progressChanged);
    },
    cancel: function(uploadId) {
      var request, upload, _i, _len, _ref;

      _ref = this.requests;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        upload = _ref[_i];
        if (upload.requestId === uploadId) {
          upload.request.abort();
        }
      }
      return this.requests = (function() {
        var _j, _len1, _ref1, _results;

        _ref1 = this.requests;
        _results = [];
        for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
          request = _ref1[_j];
          if (request.requestId !== uploadId) {
            _results.push(request);
          }
        }
        return _results;
      }).call(this);
    },
    progressChanged: function() {
      var _ref;

      if ((0 < (_ref = this.get('progress')) && _ref < 100)) {
        return this.set({
          pending: true
        });
      } else if (this.get('progress') === 100) {
        this.set({
          pending: false,
          complete: true
        });
        return this.trigger('fileupload:complete', this);
      }
    },
    requests: [],
    upload: function(file) {
      if ((file != null)) {
        this.set({
          name: file.name,
          type: file.type,
          pending: true,
          complete: false,
          file: file
        });
      }
      this.trigger('fileupload:uploadstart', this);
      return this.save();
    },
    save: function(attributes, options) {
      return this.sync("create", this, options);
    },
    sync: function(method, model, options) {
      var url,
        _this = this;

      if (method === "create") {
        if ((options != null ? options.url : void 0) == null) {
          url = this.url();
        }
        return $.ajax({
          xhr: function() {
            var xhr;

            xhr = new window.XMLHttpRequest();
            xhr.upload.addEventListener("progress", function(event) {
              model.set({
                progress: event.lengthComputable ? event.loaded / event.total : NaN
              });
              return model.trigger('change', model, {});
            }, false);
            return xhr;
          },
          type: 'POST',
          url: url,
          processData: false,
          data: (function() {
            var formData;

            formData = new window.FormData();
            formData.append("files", model.get("file"));
            return formData;
          })(),
          contentType: false
        }).done(function(data, status, jqXHR) {
          if (typeof options.success === "function") {
            options.success(data);
          }
          model.set(_.extend(model.parse(data), {
            complete: true,
            pending: false,
            progress: 100
          }));
          return model.trigger('sync', model, data, options);
        }).fail(function(jqXHR, textStatus, errorThrown) {
          if (typeof options.error === "function") {
            options.error(model, errorThrown, options);
          }
          return model.trigger('error', model, jqXHR, options);
        });
      } else {
        return Backbone.sync.apply(this, arguments);
      }
    }
  });

}).call(this);
