(function() {
  var $, Backbone, ContentTypeHelpers, LocationDialogAssetGrid, Uploader, locationsFileDialog, _;

  _ = require('underscore');

  $ = global.$;

  $ || ($ = require('jquery-browserify'));

  Backbone = require('backbone');

  Backbone.$ = $;

  ContentTypeHelpers = require('./ContentTypeHelpers.js').ContentTypeHelpers;

  LocationDialogAssetGrid = require('./LocationDialogAssetGrid.js').LocationDialogAssetGrid;

  locationsFileDialog = require('./templates.js').locationsFileDialog;

  Uploader = require('./Uploader.js').Uploader;

  exports.LocationsFileDialogView = Backbone.View.extend({
    events: {
      'click #upload-a-trigger': 'toggleUploadView'
    },
    initialize: function() {
      this.children = [];
      this.uploader = {};
      this.collection = this.options.collection;
      this.collection || (this.collection = new Backbone.Collection());
      this.template = this.options.template;
      this.template || (this.template = locationsFileDialog);
      return this.addCollectionListeners();
    },
    addCollectionListeners: function() {
      var _this = this;

      return this.listenTo(this.collection, "all", function() {
        return _this.renderChildViews();
      });
    },
    hide: function() {
      this.render();
      return this.$('#insert-image-modal', this.$el).modal('hide');
    },
    imageSelected: function(id) {
      return this.trigger('locationsFileDialogView:imageSelected', id);
    },
    render: function() {
      this.$el.html(this.template());
      this.$('#upload-files').css({
        'height': '119px;'
      });
      this.renderChildViews();
      if (this.uploader.render == null) {
        this.uploader = new Uploader({
          el: this.$('#upload-files div.well-inner')
        });
      }
      this.uploader.render();
      return this;
    },
    renderChildViews: function() {
      this.removeChildren();
      this.$('.asset-grid.editor-asset-grid', this.$el).html('<div id="gridBody"></div>');
      this.children = [
        new LocationDialogAssetGrid({
          el: this.$('#gridBody', this.$el),
          collection: this.collection
        }).render()
      ];
      return this.setupChildViewListeners();
    },
    setupChildViewListeners: function() {
      var _this = this;

      if (this.children.length > 0) {
        return _.each(this.children, function(child) {
          _this.stopListening(child, 'assetCollectionAssetGrid:imageSelected', _this.imageSelected);
          return _this.listenTo(child, 'assetCollectionAssetGrid:imageSelected', _this.imageSelected);
        });
      }
    },
    show: function() {
      this.render();
      return this.$('#insert-image-modal', this.$el).modal('show');
    },
    toggleUploadView: function(event) {
      var hiding;

      hiding = this.$('#upload-files.collapse').hasClass('in');
      if (hiding) {
        this.$('#upload-files.collapse.in').collapse('hide');
        this.collection.fetch();
      } else {
        this.$('#upload-files.collapse').collapse('show');
      }
      event.originalEvent.stopPropagation();
      event.originalEvent.preventDefault();
      event.stopPropagation();
      return event.preventDefault();
    },
    uploadDone: function() {
      return this.collection.fetch();
    },
    remove: function() {
      this.stopListening();
      this.removeChildren();
      if (this.uploader.render != null) {
        this.uploader.remove();
        this.uploader = {};
      }
      return this.$el.html('');
    },
    removeChildren: function() {
      return _.each(this.children, function(child) {
        return child.remove();
      });
    }
  });

}).call(this);
