(function() {
  var $, Backbone, FileUploads, UploadItem, uploadItem, uploader, _, _ref;

  $ = global.$;

  $ || ($ = require('jquery-browserify'));

  _ = require('underscore');

  Backbone = require('backbone');

  Backbone.$ = $;

  UploadItem = require('./UploadItem.js').UploadItem;

  _ref = require('./templates.js'), uploadItem = _ref.uploadItem, uploader = _ref.uploader;

  FileUploads = require('./FileUploads.js').FileUploads;

  exports.Uploader = Backbone.View.extend({
    addCollectionListeners: function() {
      this.stopListening(this.collection, "create add remove", this.render);
      this.listenTo(this.collection, "create add remove", this.render);
      this.stopListening(this.collection, "fileupload:complete", this.completeHandler);
      return this.listenTo(this.collection, "fileupload:complete", this.completeHandler);
    },
    dragLeave: function(event) {
      event.stopPropagation();
      event.originalEvent.stopImmediatePropagation();
      event.originalEvent.preventDefault();
      event.preventDefault();
      return this.$('.hero-unit').css({
        border: '3px dashed #ffffff'
      });
    },
    cancelDrag: function(event) {
      event.stopPropagation();
      event.originalEvent.stopImmediatePropagation();
      event.originalEvent.preventDefault();
      event.preventDefault();
      return this.$('.hero-unit').css({
        border: '3px dashed #5bb75b'
      });
    },
    highlightDrop: function(event) {
      event.stopPropagation();
      event.originalEvent.stopImmediatePropagation();
      event.originalEvent.preventDefault();
      event.preventDefault();
      return this.$('.hero-unit').css({
        border: '3px dashed #ffffff'
      });
    },
    completeHandler: function(model) {
      return this.collection.remove(model);
    },
    createFileObject: function(file) {
      var filesObj;

      filesObj = {
        file: file
      };
      return this.collection.create(_.extend(filesObj, file));
    },
    events: {
      'click #addFiles': 'selectFiles',
      'dragenter': 'highlightDrop',
      'change .uploader': 'uploadFiles',
      'dragover': 'cancelDrag',
      'dragleave': 'dragLeave',
      'drop': 'getDroppedFiles'
    },
    getDroppedFiles: function(event) {
      var _this = this;

      _.each(event.originalEvent.dataTransfer.files, function(file) {
        return _this.createFileObject(file);
      });
      event.originalEvent.stopImmediatePropagation();
      event.originalEvent.preventDefault();
      event.stopImmediatePropagation();
      event.preventDefault();
      return this.$('.hero-unit').css({
        border: '3px dashed #ffffff'
      });
    },
    initialize: function() {
      var collection, template;

      collection = this.options.collection;
      if (collection == null) {
        collection = new FileUploads();
      }
      this.collection = collection;
      template = this.options.template;
      template || (template = uploader);
      this.template = template;
      return this.addCollectionListeners();
    },
    render: function() {
      this.$el.html(this.template());
      this.renderChildViews();
      return this;
    },
    renderChildViews: function() {
      var $tbody;

      this.removeChildren();
      if (this.$(this.$el).find('tbody.files').length <= 0) {
        this.$(this.$el).find('table').append('<tbody class="files"></tbody>');
      }
      $tbody = _.head(this.$('tbody.files').get());
      this.children = this.collection.map(function(model) {
        return new UploadItem({
          model: model,
          el: $tbody,
          template: uploadItem
        }).render();
      });
      return this;
    },
    selectFiles: function(event) {
      this.undelegateEvents();
      this.$('input[type=file][multiple].uploader').remove();
      this.$el.append('<input type="file" class="uploader" multiple style="display:none"/>');
      this.delegateEvents(this.events);
      this.$('input[type=file][multiple].uploader').click();
      return event.preventDefault();
    },
    remove: function() {
      this.stopListening();
      this.removeChildren();
      this.$el.html('');
      return this;
    },
    removeChildren: function() {
      return _.each(this.children, function(view) {
        return view.remove();
      });
    },
    uploadFiles: function() {
      var _this = this;

      return _.each(_.head(this.$('input[type=file][multiple].uploader').get()).files, function(file) {
        return _this.createFileObject(file);
      });
    }
  });

}).call(this);
