chai = require('chai')
sinon = require 'sinon'
sinonChai = require 'sinon-chai'
chai.use(sinonChai)
should = chai.should()
{expect} = chai
{describe} = require './describe.coffee'
$ = require 'jquery-browserify'
_ = require 'underscore'
Backbone = require('backbone')
Backbone.$ = $
{AssetGrid} = require '../../../dist/AssetGrid.js'
{AssetCollection} = require '../../../dist/asset/model/AssetCollection.js'
{AssetModel} = require '../../../dist/asset/model/AssetModel.js'
{AssetTestData} = require './asset/spec/AssetTestData.coffee'
{AssetCollectionAssetGrid} = require '../../../dist/AssetCollectionAssetGrid.js'
{editorAssetsThumbnails} = require '../../../dist/templates.js'

describe('AssetCollectionAssetGrid',
  ->
    beforeEach (done) ->
      $('#fixtures').html('<div id="test-asset-collection-asset-grid"></div>')
      @collection = new AssetCollection().add(AssetTestData)
      @view = new AssetCollectionAssetGrid
        el: '#fixtures #test-asset-collection-asset-grid'
        collection: @collection
      done()

    afterEach (done) ->
      @view = @collection = {}
      $('#fixtures').html('')
      done()

    it 'should exist',
      (done) ->
        expect(@view).to.exist
        done()

    it 'should be a Backbone view',
      (done) ->
        @view.should.be.an.instanceof Backbone.View
        done()
        
    describe('#collection',
      ->
        it 'should exist',
          (done) ->
            expect(@view.collection).to.exist
            done()

        it 'should be @collection',
          (done) ->
            @view.collection.should.deep.equal @collection
            done()

        it 'should be a AssetCollection',
          (done) -> 
            @view.collection.should.be.an.instanceof AssetCollection
            done()

    )

    describe('#collectionClass',
      ->
        it 'should exist',
          (done) ->
            expect(@view.collectionClass).to.exist
            done()
            
        it 'should be AssetCollection',
          (done) ->
            @view.collectionClass.should.deep.equal(AssetCollection)
            done()
    )

    describe('#el'
      ->
        it 'should exist',
          (done) ->
            expect(@view.$el).to.exist
            done()
            
        it '#$el should be a div with an id of test-asset-collection-asset-grid',
          (done) -> 
            @view.$el.attr('id').should.equal 'test-asset-collection-asset-grid'
            done()
    )

    describe('#events',
      ->
        it 'should exist',
          (done) ->
            expect(@view.events).to.exist
            done()

        describe('#"click img"'
          ->
            it 'should exist',
              (done) ->
                expect(@view.events['click img']).to.exist
                done()
        )
    )

    describe('#imageClicked',
      ->
        it 'should exist',
          (done) ->
            expect(@view.imageClicked).to.exist
            done()

        it 'should dispatch an assetCollectionAssetGrid:imageSelected event with the asset id that was selected',
          (done) ->
            spy = sinon.spy()
            @view.on('assetCollectionAssetGrid:imageSelected', spy)
            event =
              target: _.head($('<img src="/assets/files/0dde0ac0-9bce-11e2-8e1a-60334b0eb8b6/thumbnail/160/130" image-id="0dde0ac0-9bce-11e2-8e1a-60334b0eb8b6" />').get())
            @view.imageClicked(event)
            spy.should.have.been.called
            done()
            
    )
                
    describe('#render',
      ->
        it 'should exist',
          (done) ->
            expect(@view.render).to.exist
            done()

        it 'should react to collection changes by rendering',
          (done) ->
            spy = sinon.spy(@view, "render")
            @view.collection.remove(@view.collection.at(0))
            spy.should.have.been.called
            @view.render.restore()
            done()

        it 'should render something',
          (done) ->
            @view.$el.html().should.not.equal ""
            done()
    )

    describe('#setupCollectionListeners',
      ->
        it 'should exist',
          (done) ->
            expect(@view.setupCollectionListeners).to.exist
            done()
    )

    describe('#template',
      ->
        it '#template should be editorAssetsThumbnails',
          (done) ->
            @view.template.should.deep.equal editorAssetsThumbnails
            done()
    )
)