$ = global.$
$ or= require 'jquery-browserify'
{describe} = require './describe.coffee'
_ = require 'underscore'
sinon = require 'sinon'
sinonChai = require 'sinon-chai'
chai = require 'chai'
chai.use(sinonChai)
should = chai.should()
{expect} = chai
Backbone = require 'backbone'
Backbone.$ = $
{FileUpload} = require '../../../dist/FileUpload.js'

describe('FileUpload'
  ->
    beforeEach (done) ->
      @upload = new FileUpload()
      done()

    afterEach (done) ->
      @upload = {}
      done()
      
    it 'should exist',
      (done) ->
        expect(FileUpload).to.exist
        done()

    describe('#destroy',
      ->
        it 'should exist',
          (done) ->
            expect(@upload.destroy).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @upload.destroy.should.be.an.instanceof Function
            done()

        it 'should fire the proper destroy event',
          (done) ->
            destroyStub = sinon.stub()
            @upload.on("destroy", destroyStub)
            @upload.destroy()
            destroyStub.should.have.been.calledWith @upload
            @upload.off("destroy", destroyStub)
            done()
    )

    describe('#url'
      ->
        it 'should exist',
          (done) ->
            expect(@upload.url).to.exist
            done()
    )
    describe('#defaults',
      ->
        beforeEach (done) ->
          @defaults =
            lastModifiedDate: ""
            name: ""
            size: 0
            type: ""
            maxFileSize: 50000000
            pending: false
            progress: 0.00
            complete: false
          done()

        afterEach (done) ->
          @defaults = {}
          done()

        describe('#name'
          ->
            it 'should exist',
              (done) ->
                @upload.has('name').should.be.true
                done()

            it 'should be the empty string',
              (done) ->
                @upload.get('name').should.equal @defaults.name
                done()
        )

        describe('#type',
          ->
            it 'should exist',
              (done) ->
                @upload.has('type').should.be.true
                done()

            it 'should be the empty string',
              (done) ->
                @upload.get('type').should.equal @defaults.type
                done()

        )

        describe('#size',
          ->
            it 'should exist',
              (done) ->
                @upload.has('size').should.be.true
                done()

            it 'should be 0',
              (done) ->
                @upload.get('size').should.equal @defaults.size
                done()
        )

        describe('#maxFileSize',
          ->
            it 'should exist',
              (done) ->
                @upload.has('maxFileSize').should.be.true
                done()

            it 'should be 500000000',
              (done) ->
                @upload.get('maxFileSize').should.equal @defaults.maxFileSize
                done()
        )

        describe('#progress',
          ->
            it 'should exist',
              (done) ->
                @upload.has('progress').should.be.true
                done()

            it 'should be 0.00',
              (done) ->
                @upload.get('progress').should.equal @defaults.progress
                done()

            it 'changing progress should trigger a change event',
              (done) ->
                spy = sinon.spy()
                @upload.on('change', spy)
                @upload.set(progress: 35)
                spy.should.have.been.calledWith(@upload)
                @upload.off('change', spy)
                done()

            it 'a positive, progress less than 100 means the model is pending',
              (done) ->
                @upload.set(progress: 10)
                @upload.get("pending").should.be.true
                done()

            it 'a progress of 100 means the upload is complete',
              (done) ->
                @upload.set(progress: 100)
                @upload.get('pending').should.be.false
                @upload.get('complete').should.be.true
                done()

            it 'should fire a fileupload:complete event when the upload is complete',
              (done) ->
                spy = sinon.spy()
                @upload.on('fileupload:complete', spy)
                @upload.set(progress: 100)
                spy.should.have.been.calledWith(@upload)
                done()
        )

        

        describe('#pending',
          ->
            it 'should exist',
              (done) ->
                @upload.has('pending').should.be.true
                done()

            it 'should be false',
              (done) ->
                @upload.get('pending').should.be.false
                done()

        )
        
        describe('#complete',
          ->
            it 'should exist',
              (done) ->
                @upload.has('complete').should.be.true
                done()

            it 'should be false',
              (done) ->
                @upload.get('complete').should.be.false
                done()
        )
    )
    describe('#cancel(uploadId)',
      ->
        beforeEach (done) ->
          @abortStub = sinon.stub()
          @xhrMock = sinon.mock(new XMLHttpRequest())
          _.extend(@xhrMock, abort: @abortStub)
          done()

        afterEach (done) ->
          @xhrMock.restore()
          @upload.requests = []
          @abortStub = {}
          @xhrMock = {}
          done()
          
        it 'should exist',
          (done) ->
            expect(@upload.cancel).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @upload.cancel.should.be.an.instanceof Function
            done()

        it 'should cancel an upload',
          (done) ->
            fakeRequestObject =
              requestId: "aabbcc"
              request: @xhrMock
            @upload.requests.push(requestId: "aabbcc", request: @xhrMock)
            @upload.cancel("aabbcc")
            _.delay(
              =>
                @upload.requests.length.should.equal 0
                done()
              1999
            )
    )

    describe('progressChanged',
      ->
        it 'should exist',
          (done) ->
            expect(@upload.progressChanged).to.exist
            done()

        it 'should set pending to true if called with less than 100 progress',
          (done) ->
            @upload.set(progress: 10)
            spy = sinon.spy(@upload, 'set')
            @upload.progressChanged()
            spy.should.have.been.calledWith pending:true
            spy.restore()
            done()

        it 'should set complete to true and pending to false when called with 100 progress',
          (done) ->
            @upload.set(progress: 100)
            spy = sinon.spy(@upload, 'set')
            @upload.progressChanged()
            spy.should.have.been.calledWith pending: false, complete: true
            spy.restore()
            done()

        it 'should fire a fileupload:complete event when called with 100 progress',
          (done) ->
            spy = sinon.spy()
            @upload.on('fileupload:complete', spy)
            @upload.set(progress: 100)
            @upload.progressChanged()
            spy.should.have.been.called
            @upload.off('fileupload:complete', spy)
            done()
    )
    
    describe('#requests',
      ->
        it 'should exist',
          (done) ->
            expect(@upload.requests).to.exist
            done()

        it 'should be an Array',
          (done) ->
            @upload.requests.should.be.an.instanceof Array
            done()
    )

    describe('#save',
      ->
        beforeEach (done) ->
          @spy = sinon.stub(@upload, 'sync')
          done()

        afterEach (done) ->
          @spy.restore()
          done()

        it 'should exist',
          (done) ->
            expect(@upload.save).to.exist
            done()

        it 'should call @upload sync with "create", @upload',
          (done) ->
            @upload.save()
            @spy.should.have.been.calledWith 'create', @upload
            done()
    )

    describe('#sync',
      ->
        beforeEach (done) ->
          @mockJquery = sinon.mock($)
          done()

        afterEach (done) ->
          @mockJquery = @mockJquery.restore()
          done()

        it 'should exist',
          (done) ->
            expect(@upload.sync).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @upload.sync.should.be.an.instanceof Function
            done()

        it 'should not be Backbone.sync',
          (done) ->
            @upload.sync.should.not.deep.equal Backbone.sync
            done()
    )

    describe("#upload(file)",
      ->
        beforeEach (done) ->
          @spy = sinon.stub(@upload, "save", ->)
          done()

        afterEach (done) ->
          @spy.restore()
          done()

        it 'should exist',
          (done) ->
            expect(@upload.upload).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @upload.upload.should.be.an.instanceof Function
            done()

        it 'should fire a fileupload:uploadstart event',
          (done) ->
            uploadStartStub = sinon.stub()
            @upload.on('fileupload:uploadstart', uploadStartStub)
            @upload.upload(
              name: "test.png"
              type: "image/png"
              data: ""
              size: 1234
            )
            uploadStartStub.should.have.been.calledWith @upload
            done()
    )
)