chai = require('chai')
sinon = require 'sinon'
sinonChai = require 'sinon-chai'
chai.use(sinonChai)
should = chai.should()
{expect} = chai
{describe} = require './describe.coffee'
$ = require 'jquery-browserify'
_ = require 'underscore'
Backbone = require('backbone')
Backbone.$ = $
{locationsFileDialog} = require '../../../dist/templates.js'
{LocationsFileDialogView} = require '../../../dist/LocationsFileDialogView.js'
{AssetCollection} = require '../../../dist/asset/model/AssetCollection.js'
{AssetCollectionAssetGrid} = require '../../../dist/AssetCollectionAssetGrid.js'
{AssetTestData} = require './asset/spec/AssetTestData.coffee'

describe('LocationsFileDialogView'
  ->
    beforeEach (done) ->
      $('#fixtures').html('<div id="test-locations-dialog-view"></div>')
      @collection = new AssetCollection().add(AssetTestData)
      @view = new LocationsFileDialogView
        el: '#fixtures #test-locations-dialog-view'
        collection: @collection
      @view.render()
      done()

    afterEach (done) ->
      @view.remove()
      $('#fixtures').html('')
      @collection = @view = {}
      done()

    it 'should exist',
      (done) ->
        expect(LocationsFileDialogView).to.exist
        done()

    it 'should be a Backbone View',
      (done) ->
        @view.should.be.an.instanceof Backbone.View
        done()

    describe('#$el',
      ->
        it 'should be the test-locations-dialog-view',
          (done) ->
            @view.$el.attr('id').should.equal 'test-locations-dialog-view'
            done()
    )


    describe('#template',
      ->
        it 'should exist',
          (done) ->
            expect(@view.template).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @view.template.should.be.an.instanceof Function
            done()

        it 'should be locationsFileDialog',
          (done) ->
            @view.template.should.deep.equal locationsFileDialog
            done()

    )

    describe('#render',
      ->
        it 'should exist',
          (done) ->
            expect(@view.render).to.exist
            done()

        it 'should return the view',
          (done) ->
            @view.render().should.deep.equal @view
            done()

        it "should render html into the view's $el'",
          (done) ->
            @view.render().$el.html().should.not.equal ''
            done()

        it 'should render at least an upload-a-trigger in the view',
          (done) ->
            $('#upload-a-trigger', @view.render().$el).length.should.be.greaterThan 0
            done()

        it 'should render its child views',
          (done) ->
            spy = sinon.spy(@view, "renderChildViews")
            @view.render()
            spy.should.have.been.called
            spy.restore()
            done()
    )

    describe('#events',
      ->
        it 'should exist',
          (done) ->
            expect(@view.events).to.exist
            done()

        describe('click #upload-a-trigger'
          ->
            it 'should exist',
              (done) ->
                expect(@view.events['click #upload-a-trigger']).to.exist
                done()

            it 'should be "toggleUploadView"',
              (done) ->
                @view.events['click #upload-a-trigger'].should.equal 'toggleUploadView'
                done()
        )
    )

    describe('#addCollectionListeners',
      ->
        it 'should exist',
          (done) ->
            expect(@view.addCollectionListeners).to.exist
            done()

        it 'should call listenTo with the collection as an arg',
          (done) ->
            spy = sinon.spy(@view, "listenTo")
            @view.addCollectionListeners()
            spy.should.have.been.calledWith(@collection, "all")
            spy.restore()
            done()

    )

    describe('#collection',
      ->
        it '#collection should exist',
          (done) ->
            expect(@view.collection).to.exist
            done()

        it '#collection should be a Backbone.Collection',
          (done) ->
            @view.collection.should.be.an.instanceof Backbone.Collection
            done()

        it '#collection should be an AssetCollection',
          (done) ->
            @view.collection.should.be.an.instanceof AssetCollection
            done()

        it 'changing the collection should call renderChildViews',
          (done) ->
            spy = sinon.spy(@view, "renderChildViews")
            @view.collection.remove(@view.collection.at(0))
            spy.should.have.been.called
            spy.restore()
            done()
    )


    describe('#children',
      ->
        it 'should exist',
          (done) ->
            expect(@view.children).to.exist
            done()

        it 'should be an Array',
          (done) ->
            @view.children.should.be.an.instanceof Array
            done()
    )

    describe('#hide',
      ->
        beforeEach (done) ->
          @modalStub = sinon.stub()
          @mockJqueryFn = sinon.mock($.fn)
          _.extend($.fn, modal: @modalStub)
          done()

        afterEach (done) ->
          @mockJqueryFn.restore()
          @mockJqueryFn = @modalStub = {}
          done()

        it 'should exist',
          (done) ->
            expect(@view.hide).to.exist
            done()

        it 'should call render',
          (done) ->
            spy = sinon.spy(@view, "render")
            @view.hide()
            spy.should.have.been.called
            spy.restore()
            done()

        it 'should hide the modal',
          (done) ->
            @view.hide()
            @modalStub.should.have.been.calledWith('hide')
            done()
    )

    describe('#imageSelected'
      ->
        it 'should exist',
          (done) ->
            expect(@view.imageSelected).to.exist
            done()

        it 'should dispatch an locationsFileDialogView:imageSelected event with the asset id that was selected',
          (done) ->
            spy = sinon.spy()
            @view.on('locationsFileDialogView:imageSelected', spy)
            @view.imageSelected("someText")
            spy.should.have.been.called
            done()
    )

    describe('#renderChildViews',
      ->
        it 'should exist',
          (done) ->
            expect(@view.renderChildViews).to.exist
            done()

        it 'should put some html into gridBody',
          (done) ->
            @view.render()
            @view.renderChildViews()
            $gridBody = @view.$('#gridBody', @view.$el)
            $gridBody.html().should.not.equal '\n        ## asset grid will be inserted here via js\n      '
            $gridBody.html().should.not.equal ''
            done()

        it 'should call setupChildViewListeners',
          (done) ->
            spy = sinon.spy(@view, "setupChildViewListeners")
            @view.render()
            spy.should.have.been.called
            spy.restore()
            done()
            
    )

    describe('#setupChildViewListeners',
      ->
        beforeEach (done) ->
          done()

        afterEach (done) ->
          done()

        it 'should exist',
          (done) ->
            expect(@view.setupChildViewListeners).to.exist
            done()

        it 'should fire imageSelected event',
          (done) ->
            spy = sinon.spy(@view, "imageSelected")
            @view.render()
            $('#fixtures').append('<div id="special-test-asset-collection-asset-grid"></div>')
            specialAssetCollectionAssetGrid = new AssetCollectionAssetGrid(el: '#fixtures #special-test-asset-collection-asset-grid', collection: @collection)
            @view.children[0] = specialAssetCollectionAssetGrid
            @view.setupChildViewListeners()
            specialAssetCollectionAssetGrid.trigger('assetCollectionAssetGrid:imageSelected', "something")
            spy.should.have.been.calledWith("something")
            spy.restore()
            done()
    )
    

    describe('show',
      ->
        beforeEach (done) ->
          @modalStub = sinon.stub()
          @mockJqueryFn = sinon.mock($.fn)
          _.extend($.fn, modal: @modalStub)
          done()

        afterEach (done) ->
          @mockJqueryFn.restore()
          @mockJqueryFn = @modalStub = {}
          done()
          
        it 'should exist',
          (done) ->
            expect(@view.show).to.exist
            done()

        it 'should render the view',
          (done) ->
            spy = sinon.spy(@view, "render")
            @view.show()
            spy.should.have.been.called
            spy.restore()
            done()

        it 'should show the modal',
          (done) ->
            @view.show()
            @modalStub.should.have.been.calledWith('show')
            done()

    )

    describe('#uploadDone'
      ->
        it 'should exist',
          (done) ->
            expect(@view.uploadDone).to.exist
            done()

        it 'should refresh the collection',
          (done) ->
            spy = sinon.spy(@view.collection, "fetch")
            @view.uploadDone()
            spy.should.have.been.called
            spy.restore()
            done()
    )

    describe('#toggleUploadView',
      ->
        beforeEach (done) ->
          @mockJqueryFn = sinon.mock($.fn)
          @collapseStub = sinon.stub()
          _.extend($.fn, collapse: @collapseStub)
          done()

        afterEach (done) ->
          @mockJqueryFn.restore()
          @mockJqueryFn = @collapseStub = @expectedFileuploadArguments = {}
          done()

        it 'should expand the collapsed upload well',
          (done) ->
            mockEvent =
              stopPropagation: ->
              preventDefault: ->
              originalEvent:
                stopPropagation: ->
                stopImmediatePropagation: ->
                preventDefault: ->
            @view.toggleUploadView(mockEvent)
            @collapseStub.should.have.been.called
            done()

    )

    describe('#uploader',
      ->
        it 'should exist',
          (done) ->
            expect(@view.uploader).to.exist
            done()
    )
)