chai = require 'chai'
sinon = require 'sinon'
sinonChai = require 'sinon-chai'
chai.use(sinonChai)
should = chai.should()
{expect} = chai
$ = global.$
$ or= require 'jquery-browserify'
_ = require 'underscore'
Backbone = require 'backbone'
Backbone.$ = $
{describe} = require './describe.coffee'
{UploadItem} = require '../../../dist/UploadItem.js'
{uploadItem} = require '../../../dist/templates.js'
{FileUpload} = require '../../../dist/FileUpload.js'

describe('UploadItem',
  ->
    beforeEach (done) ->
      $('#fixtures').html('<table class="table upload-table"><tbody class="files"></tbody></table>')
      @model = new FileUpload
        filename: "test.jpg"
        filetype: "image/jpg"
        maxFileSize: 50000000
        pending: true
        progress: 35
        complete: false
        file:
          size: 51200
          filename: "test.jpg"
      @uploadItem = new UploadItem
        model: @model
        el: $('#fixtures table.upload-table tbody.files')
      done()

    afterEach (done) ->
      @uploadItem.remove()
      @model = @uploadItem = {}
      $('#fixtures').html('')
      done()

    it 'should exist',
      (done) ->
        expect(UploadItem).to.exist
        done()

    it 'should be a Backbone.View',
      (done) ->
        @uploadItem.should.be.an.instanceof Backbone.View
        done()

    describe('#addEventListeners',
      ->
        beforeEach (done) ->
          @listenToSpy = sinon.spy(@uploadItem, "listenTo")
          @stopListeningSpy = sinon.spy(@uploadItem, "stopListening")
          done()

        afterEach (done) ->
          @stopListeningSpy.restore()
          @listenToSpy.restore()
          done()

        it 'should exist',
          (done) ->
            expect(@uploadItem.addEventListeners).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploadItem.addEventListeners.should.be.an.instanceof Function
            done()

        it 'should listen for all model events and attach render',
          (done) ->
            @uploadItem.addEventListeners()
            @listenToSpy.should.have.been.calledWith(@model, "all", @uploadItem.render)
            done()

        it 'should remove the previous listener',
          (done) ->
            @uploadItem.addEventListeners()
            @stopListeningSpy.should.have.been.calledWith(@model, "all", @uploadItem.render)
            done()

        it 'should remove the listeners before adding the new ones',
          (done) ->
            @uploadItem.addEventListeners()
            @stopListeningSpy.should.have.been.calledBefore @listenToSpy
            done()
    )

    describe('#cancelUpload',
      ->
        beforeEach (done) ->
          @abortStub = sinon.stub()
          @xhrMock = sinon.mock(new XMLHttpRequest())
          _.extend(@xhrMock, abort: @abortStub)
          done()

        afterEach (done) ->
          @xhrMock.restore()
          @xhrMock = {}
          @abortStub = {}
          @model.requests = []
          done()
          
        it 'should exist',
          (done) ->
            expect(@uploadItem.cancelUpload).to.exist
            done()
        it 'should be a Function',
          (done) ->
            @uploadItem.cancelUpload.should.be.an.instanceof Function
            done()

        it 'should call the model\'s cancelUpload method',
          (done) ->
            spy = sinon.spy(@model, "cancel")
            fakeRequestObject =
              requestId: "aabbcc"
              request: @xhrMock
            @model.requests.push(requestId: "aabbcc", request: @xhrMock)
            @uploadItem.cancelUpload()
            spy.should.have.been.calledWith("aabbcc")
            spy.restore()
            done()
    )

    describe('#events',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.events).to.exist
            done()

        describe('#click td.cancel button.btn',
          ->
            it 'should exist',
              (done) ->
                expect(@uploadItem.events['click td.cancel button.btn']).to.exist
                done()
        )

        describe('#click td.start button.btn',
          ->
            it 'should exist',
              (done) ->
                expect(@uploadItem.events['click td.start button.btn']).to.exist
                done()

            it 'should be startUpload',
              (done) ->
                @uploadItem.events['click td.start button.btn'].should.equal "startUpload"
                done()
        )
    )

    describe('#initialize()',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.initialize).to.exist
            done()

    )

    describe('#model',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.model).to.exist
            done()
    )

    describe('#render',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.render).to.exist
            done()

        it 'should put some html into $el',
          (done) ->
            @uploadItem.render()
            @uploadItem.$el.html().should.not.equal ''
            done()
    )

    describe('#remove',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.remove).to.exist
            done()
        it 'should be a Function',
          (done) ->
            @uploadItem.remove.should.be.an.instanceof Function
            done()

        it 'should remove the item from the dom',
          (done) ->
            @uploadItem.render()
            @uploadItem.remove()
            _.delay(
              =>
                $('#fixtures #' + @uploadItem.model.cid).length.should.equal 0
                done()
              1000
            )
    )

    describe('#startUpload',
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.startUpload).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploadItem.startUpload.should.be.an.instanceof Function
            done()

        it 'should call the model\'s upload method',
          (done) ->
            oldUpload = @model.upload
            @model.upload = sinon.stub()
            @uploadItem.startUpload()
            @model.upload.should.have.been.called
            @model.upload = oldUpload
            done()
    )

    describe('#template(context)'
      ->
        it 'should exist',
          (done) ->
            expect(@uploadItem.template).to.exist
            done()

        it 'should be uploadItem',
          (done) ->
            @uploadItem.template.should.deep.equal uploadItem
            done()
    )

    
)
