{describe} = require './describe.coffee'
chai = require 'chai'
sinon = require 'sinon'
sinonChai = require 'sinon-chai'
chai.use(sinonChai)
should = chai.should()
{expect} = chai
$ = global.$
$ or= 'jquery-browserify'
_ = require 'underscore'
Backbone = require 'backbone'
Backbone.$ = $
{Uploader} = require '../../../dist/Uploader.js'
{uploader} = require '../../../dist/templates.js'
{FileUpload} = require '../../../dist/FileUpload.js'

describe('Uploader',
  ->
    beforeEach (done) ->
      $('#fixtures').html('<div id="uploaderHolder"></div>')
      @uploader = new Uploader(collection: new Backbone.Collection(), el: _.head($('#fixtures #uploaderHolder').get())).render()
      done()

    afterEach (done) ->
      @uploader.remove()
      $('#fixtures').html('')
      @uploader = {}
      done()
    
    it 'should exist',
      (done) ->
        expect(Uploader).to.exist
        done()
    it 'should be an instanceof Backbone.View',
      (done) ->
        @uploader.should.be.an.instanceof Backbone.View
        done()

    describe('#addCollectionListeners()'
      ->
        beforeEach (done) ->
          @spyAdd = sinon.spy(@uploader, "listenTo")
          @spyRemove = sinon.spy(@uploader, "stopListening")
          done()

        afterEach (done) ->
          @spyAdd.restore()
          @spyRemove.restore()
          done()
          
        it 'should exist',
          (done) ->
            expect(@uploader.addCollectionListeners).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploader.addCollectionListeners.should.be.an.instanceof Function
            done()

        it 'should return uploader',
          (done) ->
            @uploader.addCollectionListeners().should.deep.equal @uploader
            done()

        it 'should add the add event listener to its collection instance',
          (done) ->
            @uploader.addCollectionListeners()
            @spyAdd.should.have.been.calledWith(@uploader.collection, "create add remove", @uploader.render)
            done()

        it 'should remove the add event listener to its collection instance',
          (done) ->
            @uploader.addCollectionListeners()
            @spyRemove.should.have.been.calledWith(@uploader.collection, "create add remove", @uploader.render)
            done()

        it 'should remove the add event listener before adding the add event listener',
          (done) ->
            @uploader.addCollectionListeners()
            @spyRemove.should.have.been.calledBefore(@spyAdd)
            done()
    )

    describe('cancelDrag',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.cancelDrag).to.exist
            done()
        it 'should be a Function',
          (done) ->
            @uploader.cancelDrag.should.be.an.instanceof Function
            done()

        it 'should cancel the event passed to it and change the drop area color back to white',
          (done) ->
            @uploader.render()
            stopPropagationSpy = sinon.spy()
            preventDefaultSpy = sinon.spy()
            mockEvent =
              originalEvent:
                stopImmediatePropagation: stopPropagationSpy
                preventDefault: preventDefaultSpy
              stopPropagation: stopPropagationSpy
              preventDefault: preventDefaultSpy
              
            @uploader.cancelDrag(mockEvent)
            stopPropagationSpy.should.have.been.called
            preventDefaultSpy.should.have.been.called
            done()
    )

    describe('#collection',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.collection).to.exist
            done()
    )

    describe('#completeHandler',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.completeHandler).to.exist
            done()

        it 'should remove the model passed to it from its collection',
          (done) ->
            testModel = new FileUpload()
            @uploader.collection.add testModel
            expect(@uploader.collection.get(testModel.cid)).to.exist
            @uploader.completeHandler testModel
            expect(@uploader.collection.get(testModel.cid)).to.not.exist
            done()
        it 'should be called on collection fileupload:complete',
          (done) ->
            spy = sinon.spy(@uploader, 'completeHandler')
            testModel = new FileUpload()
            @uploader.collection.add testModel
            @uploader.addCollectionListeners()
            testModel.trigger('fileupload:complete', testModel)
            spy.should.have.been.calledWith(testModel)
            spy.restore()
            done()
    )

    describe('dragLeave',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.dragLeave).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploader.dragLeave.should.be.an.instanceof Function
            done()
    )

    describe('#events',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.events).to.exist
            done()

        describe('#click #addFiles'
          ->
            it 'should exist',
              (done) ->
                expect(@uploader.events['click #addFiles']).to.exist
                done()

            it 'should be "selectFiles"',
              (done) ->
                @uploader.events['click #addFiles'].should.equal 'selectFiles'
                done()
        )

        describe('#dragenter'
          ->
            it 'should exist',
              (done) ->
                expect(@uploader.events.dragenter).to.exist
                done()

            it 'should be highlightDrop',
              (done) ->
                @uploader.events.dragenter.should.equal 'highlightDrop'
                done()
        )

        describe('#dragleave',
          ->
            it 'should exist',
              (done) ->
                expect(@uploader.events.dragleave).to.exist
                done()

            it 'should be dragLeave',
              (done) ->
                @uploader.events.dragleave.should.equal 'dragLeave'
                done()
        )

        describe('#dragover',
          ->
            it 'should exist',
              (done) ->
                expect(@uploader.events.dragover).to.exist
                done()

            it 'should be cancelDrag',
              (done) ->
                @uploader.events.dragover.should.equal 'cancelDrag'
                done()
        )

        describe('drop',
          ->
            it 'should exist',
              (done) ->
                expect(@uploader.events.drop).to.exist
                done()

            it 'should be getDroppedFiles',
              (done) ->
                @uploader.events.drop.should.equal 'getDroppedFiles'
                done()

        )
    )

    describe('#getDroppedFiles(event)',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.getDroppedFiles).to.exist
            done()

        it 'should be a function',
          (done) ->
            @uploader.getDroppedFiles.should.be.an.instanceof Function
            done()

        it 'should call create on its collection with the files',
          (done) ->
            mockEvent =
              originalEvent:
                dataTransfer:
                  files: [filename: "test.png", filetype: "image/png"]
                stopImmediatePropagation: ->
                preventDefault: ->
              stopImmediatePropagation: ->
              preventDefault: ->
            createSpy = sinon.spy()
            repColClass = Backbone.Collection.extend(create: createSpy)            
            oldCollection = @uploader.collection
            @uploader.collection = new repColClass()
            @uploader.getDroppedFiles(mockEvent)
            createSpy.should.have.been.calledWith(filename: "test.png", filetype: "image/png", file: _.head(mockEvent.originalEvent.dataTransfer.files))
            @uploader.collection = oldCollection
            done()
    )

    describe('#highlightDrop'
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.highlightDrop).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploader.highlightDrop.should.be.an.instanceof Function
            done()

        it 'should highlight the drop area so that people know they can drop the files',
          (done) ->
            @uploader.render()
            stopPropagationSpy = sinon.spy()
            preventDefaultSpy = sinon.spy()
            mockEvent =
              originalEvent:
                stopImmediatePropagation: stopPropagationSpy
                preventDefault: preventDefaultSpy
              stopPropagation: stopPropagationSpy
              preventDefault: preventDefaultSpy
            @uploader.highlightDrop(mockEvent)
            stopPropagationSpy.should.have.been.called
            preventDefaultSpy.should.have.been.called
            done()

    )

    describe('#render()',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.render).to.exist
            done()

        it 'should be called when an item is added to the collection',
          (done) ->
            spy = sinon.spy(@uploader, "render")
            # necessary because listenTo uses a reference, not the name
            @uploader.addCollectionListeners()
            @uploader.collection.add(filename: "test.png", filetype: "image/png", file:{size: 51200})
            spy.should.have.been.called
            spy.restore()
            done()

        it 'should render some html into its el',
          (done) ->
            @uploader.render()
            @uploader.$el.html().should.not.equal ''
            done()

        it 'should call renderChildViews',
          (done) ->
            spy = sinon.spy(@uploader, "renderChildViews")
            @uploader.render()
            spy.should.have.been.called
            spy.restore()
            done()
    )

    describe('#renderChildViews',
      ->
        beforeEach (done) ->
          @uploader.collection.add(filename: "test.png", filetype: "image/png", file:{size: 51200})
          done()

        afterEach (done) ->
          @uploader.collection.remove(@uploader.collection.at(0))
          done()
          
        it 'should exist',
          (done) ->
            expect(@uploader.renderChildViews).to.exist
            done()

        it 'should add at least one td to the table body added by render',
          (done) ->
            @uploader.$('td', @uploader.$el).length.should.be.greaterThan 0
            done()
    )

    

    describe('#selectFiles',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.selectFiles).to.exist
            done()

        it 'should be a Function',
          (done) ->
            @uploader.selectFiles.should.be.an.instanceof Function
            done()

        it 'should open the file viewer',
          (done) ->
            # mocking or stubbing the entire
            # browser file api isn't feasible
            # in PhantomJS. We assume <input>.click()
            # works in a browser and that it
            # opens the file viewer
            # on the user's OS. This
            # test is here to tell
            # future developers what
            # Uploader#selectFiles does -- it allows
            # the user to select files when
            # called.
            true.should.equal.true
            done()
    )

    describe('#template(context)',
      ->
        it 'should exist',
          (done) ->
            expect(@uploader.template).to.exist
            done()

        it 'should be a function',
          (done) ->
            @uploader.template.should.be.an.instanceof Function
            done()

        it 'should be uploader',
          (done) ->
            @uploader.template.should.deep.equal uploader
            done()

    )
)
